#!/bin/bash

# Ensure the script is run with exactly two arguments
if [ $# -ne 2 ]; then
    echo "Usage: $0 <path> <name>"
    exit 1
fi

# Define variables
path="$1"
name="$2"

# Create the folder structure
mkdir -p "$path/$name/_werc" "$path/$name/blog/_werc"

# Create the first config file (not under blog)
cat > "$path/$name/_werc/config" <<EOF
extraHeaders='<link rel="stylesheet" type="text/css" href="_werc/style.css">'
conf_enable_wiki
conf_enable_cssedit $name
css_file='_werc/style.css'
EOF

# Create the second config file (under blog)
cat > "$path/$name/blog/_werc/config" <<EOF
conf_enable_wiki
conf_enable_blog
conf_blog_only_pull=0
conf_blog_editors=$name
blogTitle='$name Feed'
blogDesc=''
EOF

# Create the style.css file
echo "/* Put custom styles here */" > "$path/$name/_werc/style.css"

# Update index.html
index_file="$path/index.html"

if [ -f "$index_file" ]; then
    awk -v name="$name" '
    {
        print
        if ($0 ~ /<!-- BUTTONS -->/) {
            print "    <div class=\"grid-item\">"
            print "        <a href=\"/" name "/\"><img src=\"/_werc/pub/img/but/" name ".gif\" alt=\"" name "\" /></a>"
            print "    </div>"
            print ""
        }
    }' "$index_file" > "${index_file}.tmp" && mv "${index_file}.tmp" "$index_file"
    echo "Updated index.html with new user entry"
else
    echo "Warning: index.html not found at $index_file"
fi

# Output a success message
echo "Folder structure created successfully at $path/$name"
